
/*
 * Game
 *
 * Thomas David Baker, 2002-03-10
 */

package net.bluebones.tictactoe;

public class Game {

    // Internal representation of the board for this game.
    private byte[][] _board = new byte[3][3];
    
    // Initialize the board.
    {
        for (int x = 0; x < _board.length; x++) {
            for (int y = 0; y < _board[0].length; y++) {
                _board[x][y] = TicTacToe.EMPTY;
            }
        }    
    }
    
    // Internal representation of the current player.  Noughts starts.
    private byte _player = TicTacToe.NOUGHTS;
    // Internal array of both players.
    private Player[] _players = new Player[2];
    
    /**
     * Creates a new <code>Game</code> with the specified players.
     *
     * @param   players <code>Player[]</code> of the two players.
     * @throws          <code>IllegalArgumentException</code> unchecked, if 
     *                  there are not 2 players.
     */
    public Game(Player[] players) {
        if (players.length != 2) {
            throw new IllegalArgumentException("Game must be instantiated "
                + "with two players");
        }
        _players = players;
    }
    
    /** Called to begin a <code>Game</code>. */
    public void play() {
        play(null);
    }
    
    // main guts of the game
    private void play(Move prev) {
        
        System.out.println();
        Board.printBoard(_board);
        System.out.println();
        
        if (_player == TicTacToe.NOUGHTS) {
            System.out.print("Noughts (" 
                + _players[_player].getClass().getName() + ")");
        } else if (_player == TicTacToe.CROSSES) {
            System.out.print("Crosses (" 
                + _players[_player].getClass().getName() + ")");
        }
        System.out.println(" to move");
        System.out.println();
        
        // new board here else players can alter the game board as they wish!
        byte[][] b = Board.copy(_board);
        Move move = _players[_player].getMove(b, prev, _player);
        
        if (move == null) {
            throw new NullPointerException("Player supplied null move");
        }
        
        // check square is empty
        if (! Board.isEmpty(_board, move)) {
            System.out.println("That square is not empty");
            play(prev);
        }
        
        // place piece
        if (! Board.placePiece(_board, move)) {
            System.out.println("Illegal Move - try again");
            play(prev);
        }
        
        System.out.println("Moved at (" + move.x + ", " + move.y + ")");
        
        if (Board.isWinner(_board, _player)) {
            endOfGame(_player);
        }
        
        // change player
        _player = TicTacToe.reverse(_player);
        
        if (Board.isFull(_board)) {
            endOfGame(TicTacToe.EMPTY);
        } else {
            play(move);
        }
        
    }
    
    // Little display at the end of the game.
    private void endOfGame(byte winner) {
        
        System.out.println();
        Board.printBoard(_board);
        System.out.println();
        if (winner == TicTacToe.EMPTY) {
            System.out.println();
            System.out.println("Game ends in a draw");
        }

        
        System.exit(0);
    }
}
